package giardini;

import java.io.*;
import java.util.*;

public abstract class Pianta
{
	static private Map<String, String>datiInizializzazione = new HashMap<String, String>();

					private double consumoAcqua;//litri
					private double consumoFertilizzante;//sacchi
					private double minSole, maxSole;//ore

					private double sup;//superficie in mq
					private double hCorrente;//altezza in m

					Pianta(double ca, double cf, double minS, double maxS, double sup, double hc)
					{
						// con tutti i controlli del caso...
						// e' importante ricordare che l'unico modo sensato di far fallire un
						// costruttore e' lanciare un eccezione in modo che l'esecuzione non
						// si completi e non sia creata l'istanza.
						consumoAcqua = ca;
						consumoFertilizzante = cf;
						minSole = minS;
						maxSole = maxS;
						sup = sup;
						hCorrente = hc;
					}

					public final double getConsumoAcqua()
					{
						return consumoAcqua;
					}

					public final double getConsumoFertilizzante()
					{
						return consumoFertilizzante;
					}

					public final double getMinSole()
					{
						return minSole;
					}

					public final double getMaxSole()
					{
						return maxSole;
					}

					final double setConsumoAcqua(double ca)
					{
						if(ca < 0)
						{
							throw new IllegalArgumentException("Consumo acqua minore di zero");
						}
						double retval = consumoAcqua;
						consumoAcqua = ca;
						return retval;
					}

					final double setConsumoFertilizzante(double cf)
					{
						//inserire controlli se opportuno
						double retval = consumoFertilizzante;
						consumoFertilizzante = cf;
						return retval;
					}

					final double setMinSole(double ms)
					{
						//inserire controlli se opportuno
						double retval = minSole;
						minSole = ms;
						return retval;
					}

					final double setMaxSole(double ms)
					{
						double retval =  maxSole;
						maxSole = ms;
						return retval;
					}

					public double getSuperficie()
					{
						return sup;
					}

					protected double setSuperficie(double s)
					{
						double retval = sup;
						sup = s;
						return retval;
					}

					public abstract double getHMax();
					protected abstract double setHMax(double d);

					public double getHCorrente()
					{
						return hCorrente;
					}

					double setHCorrente(double hc)
					{
						double retval = hCorrente;
						hCorrente = hc;
						return retval;
					}

					public String toString()
					{
						return "questa pianta e' un " + getClass().getName() + "; ";
					}

					protected static String getInitData(String diChi)
					{
						String retval = datiInizializzazione.get(diChi);
						if(retval == null)
						{
							readInitData();
							retval = datiInizializzazione.get(diChi);
						}
						return retval;
					}

					static
					{
						System.out.println("Prima di ogni altra cosa");
						readInitData();
					}

					private static void readInitData()
					{
						try
						{
							BufferedReader br = new BufferedReader(new FileReader("datiInizializzazione.txt"));
							String key = null;
							String value = null;
							while( (key = br.readLine()) != null)
							{
								//ho consumato la riga che contiene l'informazione sulla tipologia di giardino
								key = br.readLine();
								value = br.readLine();
								System.out.println("key <" + key + "> value <" + value + ">");
								datiInizializzazione.put(key, value);
							}
							br.close();
						}
						catch(IOException ioe)
						{
							ioe.printStackTrace();
							System.exit(1);
						}
					}
				}
